@@ -473,12 +473,15 @@ process_desktop_file (ProcessApplicationData  *data,
 		tracker_sparql_builder_object (sparql, "nfo:SoftwareCategory");
 
 		if (icon) {
+			gchar *escaped_icon;
 			gchar *icon_uri;
 
 			/* Sanitize icon */
 			g_strstrip (icon);
 
-			icon_uri = g_strdup_printf (THEME_ICON_URN_PREFIX "%s", icon);
+			escaped_icon = g_uri_escape_string (icon, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, FALSE);
+
+			icon_uri = g_strdup_printf (THEME_ICON_URN_PREFIX "%s", escaped_icon);
 
 			tracker_sparql_builder_subject_iri (sparql, icon_uri);
 			tracker_sparql_builder_predicate (sparql, "a");
@@ -489,6 +492,7 @@ process_desktop_file (ProcessApplicationData  *data,
 			tracker_sparql_builder_object_iri (sparql, icon_uri);
 
 			g_free (icon_uri);
+			g_free (escaped_icon);
 			g_free (icon);
 		}
 
@@ -670,12 +674,15 @@ process_desktop_file (ProcessApplicationData  *data,
 			icon = g_key_file_get_string (key_file, GROUP_DESKTOP_ENTRY, "Icon", NULL);
 
 			if (icon) {
+				gchar *escaped_icon;
 				gchar *icon_uri;
 
 				/* Sanitize icon */
 				g_strstrip (icon);
 
-				icon_uri = g_strdup_printf (THEME_ICON_URN_PREFIX "%s", icon);
+				escaped_icon = g_uri_escape_string (icon, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, FALSE);
+
+				icon_uri = g_strdup_printf (THEME_ICON_URN_PREFIX "%s", escaped_icon);
 
 				tracker_sparql_builder_subject_iri (sparql, icon_uri);
 				tracker_sparql_builder_predicate (sparql, "a");
@@ -686,6 +693,7 @@ process_desktop_file (ProcessApplicationData  *data,
 				tracker_sparql_builder_object_iri (sparql, icon_uri);
 
 				g_free (icon_uri);
+				g_free (escaped_icon);
 				g_free (icon);
 			}
 		}
@@ -811,10 +819,22 @@ process_file_cb (GObject      *object,
 
 	if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY) {
 		process_directory (data, file_info, &error);
-	} else if (data->key_file) {
-		process_desktop_file (data, file_info, &error);
 	} else {
-		error = g_error_new_literal (miner_applications_error_quark, 0, "File is not a key file");
+		data->key_file = get_desktop_key_file (file, &data->type, &error);
+		if (!data->key_file) {
+			gchar *uri;
+
+			uri = g_file_get_uri (file);
+			g_warning ("Couldn't properly parse desktop file '%s': '%s'",
+			           uri,
+			           error ? error->message : "unknown error");
+			g_free (uri);
+			g_clear_error (&error);
+
+			error = g_error_new_literal (miner_applications_error_quark, 0, "File is not a key file");
+		} else {
+			process_desktop_file (data, file_info, &error);
+		}
 	}
 
 	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, error);
@@ -836,19 +856,13 @@ miner_applications_process_file (TrackerMinerFS       *fs,
                                  GCancellable         *cancellable)
 {
 	ProcessApplicationData *data;
-	GKeyFile *key_file;
 	const gchar *attrs;
-	gchar *type;
-
-	key_file = get_desktop_key_file (file, &type, NULL);
 
 	data = g_slice_new0 (ProcessApplicationData);
 	data->miner = g_object_ref (fs);
 	data->sparql = g_object_ref (sparql);
 	data->file = g_object_ref (file);
 	data->cancellable = g_object_ref (cancellable);
-	data->key_file = key_file;
-	data->type = type;
 
 	attrs = G_FILE_ATTRIBUTE_TIME_MODIFIED ","
 		G_FILE_ATTRIBUTE_STANDARD_TYPE;
