@@ -75,6 +75,8 @@ typedef struct {
 	gboolean force_internal_extractors;
 	gboolean disable_summary_on_finalize;
 
+	gchar *force_module;
+
 	gint unhandled_count;
 } TrackerExtractPrivate;
 
@@ -84,6 +86,7 @@ typedef struct {
 	GAsyncResult *res;
 	gchar *file;
 	gchar *mimetype;
+	gchar *graph;
 
 	TrackerMimetypeInfo *mimetype_handlers;
 
@@ -149,6 +152,10 @@ tracker_extract_finalize (GObject *object)
 
 	/* FIXME: Shutdown modules? */
 
+	if (priv->task_mutex) {
+		g_mutex_free (priv->task_mutex);
+	}
+
 	g_hash_table_destroy (priv->single_thread_extractors);
 	g_thread_pool_free (priv->thread_pool, TRUE, FALSE);
 
@@ -229,6 +236,7 @@ tracker_extract_new (gboolean     disable_shutdown,
 
 	priv->disable_shutdown = disable_shutdown;
 	priv->force_internal_extractors = force_internal_extractors;
+	priv->force_module = g_strdup (force_module);
 
 	return object;
 }
@@ -271,13 +279,11 @@ notify_task_finish (TrackerExtractTask *task,
 }
 
 static gboolean
-get_file_metadata (TrackerExtractTask     *task,
-                   TrackerSparqlBuilder  **preupdate_out,
-                   TrackerSparqlBuilder  **statements_out,
-                   gchar                 **where_out)
+get_file_metadata (TrackerExtractTask  *task,
+                   TrackerExtractInfo **info_out)
 {
-	TrackerSparqlBuilder *statements, *preupdate;
-	GString *where;
+	TrackerExtractInfo *info;
+	GFile *file;
 	gchar *mime_used = NULL;
 #ifdef HAVE_LIBSTREAMANALYZER
 	gchar *content_type = NULL;
@@ -286,28 +292,28 @@ get_file_metadata (TrackerExtractTask     *task,
 
 	g_debug ("Extracting...");
 
-	*preupdate_out = NULL;
-	*statements_out = NULL;
-	*where_out = NULL;
+	*info_out = NULL;
 
-	/* Create sparql builders to send back */
-	preupdate = tracker_sparql_builder_new_update ();
-	statements = tracker_sparql_builder_new_embedded_insert ();
-	where = g_string_new ("");
+	file = g_file_new_for_uri (task->file);
+	info = tracker_extract_info_new (file, task->mimetype, task->graph);
+	g_object_unref (file);
 
 #ifdef HAVE_LIBSTREAMANALYZER
+	/* FIXME: This entire section is completely broken,
+	 * it doesn't even build these days. It should be removed or fixed.
+	 * -mr (05/09/11)
+	 */
 	if (!priv->force_internal_extractors) {
 		g_debug ("  Using libstreamanalyzer...");
 
-		tracker_topanalyzer_extract (uri, statements, &content_type);
+		tracker_topanalyzer_extract (task->file, statements, &content_type);
 
 		if (tracker_sparql_builder_get_length (statements) > 0) {
 			g_free (content_type);
 			tracker_sparql_builder_insert_close (statements);
 
-			*preupdate_out = preupdate;
-			*statements_out = statements;
-			*where_out = g_string_free (where, FALSE);
+			*info_out = info;
+
 			return TRUE;
 		}
 	} else {
@@ -335,10 +341,13 @@ get_file_metadata (TrackerExtractTask     *task,
 	 */
 	if (mime_used) {
 		if (task->cur_func) {
+			TrackerSparqlBuilder *statements;
+
 			g_debug ("  Using %s...", g_module_name (task->cur_module));
 
-			(task->cur_func) (task->file, mime_used, preupdate, statements, where);
+			(task->cur_func) (info);
 
+			statements = tracker_extract_info_get_metadata_builder (info);
 			items = tracker_sparql_builder_get_length (statements);
 
 			if (items > 0) {
@@ -353,9 +362,7 @@ get_file_metadata (TrackerExtractTask     *task,
 		g_free (mime_used);
 	}
 
-	*preupdate_out = preupdate;
-	*statements_out = statements;
-	*where_out = g_string_free (where, FALSE);
+	*info_out = info;
 
 	if (items == 0) {
 		g_debug ("No extractor or failed");
@@ -391,37 +398,46 @@ static TrackerExtractTask *
 extract_task_new (TrackerExtract *extract,
                   const gchar    *uri,
                   const gchar    *mimetype,
+                  const gchar    *graph,
                   GCancellable   *cancellable,
-                  GAsyncResult   *res)
+                  GAsyncResult   *res,
+                  GError        **error)
 {
 	TrackerExtractTask *task;
+	gchar *mimetype_used;
 
-	if (!mimetype) {
+	if (!mimetype || !*mimetype) {
 		GFile *file;
 		GFileInfo *info;
+		GError *internal_error = NULL;
 
 		file = g_file_new_for_uri (uri);
 		info = g_file_query_info (file,
 		                          G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
 		                          G_FILE_QUERY_INFO_NONE,
-		                          NULL, NULL);
+		                          NULL,
+		                          &internal_error);
 
-		if (info) {
-			mimetype = g_strdup (g_file_info_get_content_type (info));
-			g_object_unref (info);
-		} else {
-			g_warning ("Could not get mimetype for '%s'", uri);
+		g_object_unref (file);
+
+		if (internal_error) {
+			g_propagate_error (error, internal_error);
 			return NULL;
 		}
 
-		g_object_unref (file);
+		mimetype_used = g_strdup (g_file_info_get_content_type (info));
+		g_debug ("Guessing mime type as '%s'", mimetype);
+		g_object_unref (info);
+	} else {
+		mimetype_used = g_strdup (mimetype);
 	}
 
 	task = g_slice_new0 (TrackerExtractTask);
 	task->cancellable = (cancellable) ? g_object_ref (cancellable) : NULL;
 	task->res = (res) ? g_object_ref (res) : NULL;
 	task->file = g_strdup (uri);
-	task->mimetype = g_strdup (mimetype);
+	task->mimetype = mimetype_used;
+	task->graph = g_strdup (graph);
 	task->extract = extract;
 
 	if (task->cancellable) {
@@ -450,19 +466,68 @@ extract_task_free (TrackerExtractTask *task)
 		g_object_unref (task->cancellable);
 	}
 
-	g_free (task->file);
+	if (task->mimetype_handlers) {
+		tracker_mimetype_info_free (task->mimetype_handlers);
+	}
+
+	g_free (task->graph);
 	g_free (task->mimetype);
+	g_free (task->file);
+
 	g_slice_free (TrackerExtractTask, task);
 }
 
+static gboolean
+filter_module (TrackerExtract *extract,
+               GModule        *module)
+{
+	TrackerExtractPrivate *priv;
+	gchar *module_basename, *filter_name;
+	gboolean filter;
+
+	priv = TRACKER_EXTRACT_GET_PRIVATE (extract);
+
+	if (!priv->force_module) {
+		return FALSE;
+	}
+
+	/* Module name is the full path to it */
+	module_basename = g_path_get_basename (g_module_name (module));
+
+	if (g_str_has_prefix (priv->force_module, "lib") &&
+	    g_str_has_suffix (priv->force_module, "." G_MODULE_SUFFIX)) {
+		filter_name = g_strdup (priv->force_module);
+	} else {
+		filter_name = g_strdup_printf ("libextract-%s.so",
+		                               priv->force_module);
+	}
+
+	filter = strcmp (module_basename, filter_name) != 0;
+
+	if (filter) {
+		g_debug ("Module filtered out '%s' (due to --force-module='%s')",
+		         module_basename,
+		         filter_name);
+	} else {
+		g_debug ("Module used '%s' (due to --force-module='%s')",
+		         module_basename,
+		         filter_name);
+	}
+
+	g_free (module_basename);
+	g_free (filter_name);
+
+	return filter;
+}
+
 static gboolean
 get_metadata (TrackerExtractTask *task)
 {
 	TrackerExtractInfo *info;
-	TrackerSparqlBuilder *preupdate, *statements;
+	TrackerSparqlBuilder *preupdate, *postupdate, *statements;
 	gchar *where = NULL;
 
-	preupdate = statements = NULL;
+	preupdate = postupdate = statements = NULL;
 
 #ifdef THREAD_ENABLE_TRACE
 	g_debug ("Thread:%p --> File:'%s' - Extracted",
@@ -477,18 +542,16 @@ get_metadata (TrackerExtractTask *task)
 		                                 "Extraction of '%s' was cancelled",
 		                                 task->file);
 
+		g_simple_async_result_complete_in_idle ((GSimpleAsyncResult *) task->res);
 		extract_task_free (task);
 		return FALSE;
 	}
 
-	if (get_file_metadata (task, &preupdate, &statements, &where)) {
-		info = tracker_extract_info_new ((preupdate) ? tracker_sparql_builder_get_result (preupdate) : NULL,
-		                                 (statements) ? tracker_sparql_builder_get_result (statements) : NULL,
-		                                 where);
-
+	if (!filter_module (task->extract, task->cur_module) &&
+	    get_file_metadata (task, &info)) {
 		g_simple_async_result_set_op_res_gpointer ((GSimpleAsyncResult *) task->res,
 		                                           info,
-		                                           (GDestroyNotify) tracker_extract_info_free);
+		                                           (GDestroyNotify) tracker_extract_info_unref);
 
 		g_simple_async_result_complete_in_idle ((GSimpleAsyncResult *) task->res);
 		extract_task_free (task);
@@ -497,6 +560,10 @@ get_metadata (TrackerExtractTask *task)
 			g_object_unref (preupdate);
 		}
 
+		if (postupdate) {
+			g_object_unref (postupdate);
+		}
+
 		if (statements) {
 			g_object_unref (statements);
 		}
@@ -666,11 +733,13 @@ void
 tracker_extract_file (TrackerExtract      *extract,
                       const gchar         *file,
                       const gchar         *mimetype,
+                      const gchar         *graph,
                       GCancellable        *cancellable,
                       GAsyncReadyCallback  cb,
                       gpointer             user_data)
 {
 	GSimpleAsyncResult *res;
+	GError *error = NULL;
 	TrackerExtractTask *task;
 
 	g_return_if_fail (TRACKER_IS_EXTRACT (extract));
@@ -685,14 +754,20 @@ tracker_extract_file (TrackerExtract      *extract,
 
 	res = g_simple_async_result_new (G_OBJECT (extract), cb, user_data, NULL);
 
-	task = extract_task_new (extract, file, mimetype, cancellable, G_ASYNC_RESULT (res));
+	task = extract_task_new (extract, file, mimetype, graph,
+	                         cancellable, G_ASYNC_RESULT (res), &error);
 
-	if (task) {
+	if (error) {
+		g_warning ("Could not get mimetype, %s", error->message);
+		g_simple_async_result_set_from_error (res, error);
+		g_simple_async_result_complete_in_idle (res);
+		g_error_free (error);
+	} else {
 		g_idle_add ((GSourceFunc) dispatch_task_cb, task);
-
-		/* task takes a ref */
-		g_object_unref (res);
 	}
+
+	/* Task takes a ref and if this fails, we want to unref anyway */
+	g_object_unref (res);
 }
 
 void
@@ -700,57 +775,84 @@ tracker_extract_get_metadata_by_cmdline (TrackerExtract *object,
                                          const gchar    *uri,
                                          const gchar    *mime)
 {
-	TrackerSparqlBuilder *statements, *preupdate;
-	gchar *where;
+	GError *error = NULL;
 	TrackerExtractPrivate *priv;
 	TrackerExtractTask *task;
-	TrackerExtractInitFunc init_func;
+	TrackerExtractInfo *info;
+	gboolean no_modules = TRUE;
 
 	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
 	priv->disable_summary_on_finalize = TRUE;
 
 	g_return_if_fail (uri != NULL);
 
-	g_message ("Extracting...");
+	task = extract_task_new (object, uri, mime, NULL, NULL, NULL, &error);
 
-	task = extract_task_new (object, uri, mime, NULL, NULL);
+	if (error) {
+		g_printerr ("Extraction failed, %s\n", error->message);
+		g_error_free (error);
 
-	if (!task) {
 		return;
 	}
 
-	task->cur_module = tracker_extract_module_manager_get_for_mimetype (task->mimetype, &init_func, NULL, &task->cur_func);
+	task->mimetype_handlers = tracker_extract_module_manager_get_mimetype_handlers (task->mimetype);
+	task->cur_module = tracker_mimetype_info_get_module (task->mimetype_handlers, &task->cur_func, NULL);
 
-	if (init_func) {
-		TrackerModuleThreadAwareness ignore;
+	while (task->cur_module && task->cur_func) {
+		if (!filter_module (object, task->cur_module) &&
+		    get_file_metadata (task, &info)) {
+			const gchar *preupdate_str, *postupdate_str, *statements_str, *where;
+			TrackerSparqlBuilder *builder;
 
-		/* Initialize module for this single run */
-		(init_func) (&ignore, NULL);
+			no_modules = FALSE;
+			preupdate_str = statements_str = postupdate_str = NULL;
+
+			builder = tracker_extract_info_get_metadata_builder (info);
+
+			if (tracker_sparql_builder_get_length (builder) > 0) {
+				statements_str = tracker_sparql_builder_get_result (builder);
+			}
+
+			builder = tracker_extract_info_get_preupdate_builder (info);
+
+			if (tracker_sparql_builder_get_length (builder) > 0) {
+				preupdate_str = tracker_sparql_builder_get_result (builder);
+			}
+
+			builder = tracker_extract_info_get_postupdate_builder (info);
+
+			if (tracker_sparql_builder_get_length (builder) > 0) {
+				postupdate_str = tracker_sparql_builder_get_result (builder);
+			}
+
+			where = tracker_extract_info_get_where_clause (info);
+
+			g_print ("\n");
+
+			g_print ("SPARQL pre-update:\n--\n%s--\n\n",
+			         preupdate_str ? preupdate_str : "");
+			g_print ("SPARQL item:\n--\n%s--\n\n",
+			         statements_str ? statements_str : "");
+			g_print ("SPARQL where clause:\n--\n%s--\n\n",
+			         where ? where : "");
+			g_print ("SPARQL post-update:\n--\n%s--\n\n",
+			         postupdate_str ? postupdate_str : "");
+
+			tracker_extract_info_unref (info);
+			break;
+		} else {
+			if (!tracker_mimetype_info_iter_next (task->mimetype_handlers)) {
+				break;
+			}
+
+			task->cur_module = tracker_mimetype_info_get_module (task->mimetype_handlers,
+			                                                     &task->cur_func,
+			                                                     NULL);
+		}
 	}
 
-	if (get_file_metadata (task, &preupdate, &statements, &where)) {
-		const gchar *preupdate_str, *statements_str;
-
-		preupdate_str = statements_str = NULL;
-
-		if (tracker_sparql_builder_get_length (statements) > 0) {
-			statements_str = tracker_sparql_builder_get_result (statements);
-		}
-
-		if (tracker_sparql_builder_get_length (preupdate) > 0) {
-			preupdate_str = tracker_sparql_builder_get_result (preupdate);
-		}
-
-		g_print ("SPARQL pre-update:\n%s\n",
-		         preupdate_str ? preupdate_str : "");
-		g_print ("SPARQL item:\n%s\n",
-		         statements_str ? statements_str : "");
-		g_print ("SPARQL where clause:\n%s\n",
-		         where ? where : "");
-
-		g_object_unref (statements);
-		g_object_unref (preupdate);
-		g_free (where);
+	if (no_modules) {
+		g_print ("No modules found to handle metadata extraction\n\n");
 	}
 
 	extract_task_free (task);
