@@ -303,6 +303,11 @@ test_xmp_apply (void)
 	 */
 
 	/* g_assert_cmpint (tracker_sparql_builder_get_length (metadata), ==, 50); */
+        g_string_free (where, TRUE);
+        g_object_unref (metadata);
+        g_object_unref (preupdate);
+
+        tracker_xmp_free (data);
 }
 
 static void
@@ -390,6 +395,8 @@ test_xmp_regions (void)
 
 	data = tracker_xmp_new (contents, size, "test://file");
 
+        g_free (contents);
+
 	g_assert_cmpint (2, ==, g_slist_length (data->regions));
 
 	region = g_slist_nth_data (data->regions, 1);
@@ -435,6 +442,8 @@ test_xmp_regions_quill (void)
 
 	data = tracker_xmp_new (contents, size, "test://file");
 
+        g_free (contents);
+
 	g_assert_cmpint (2, ==, g_slist_length (data->regions));
 
 	region = g_slist_nth_data (data->regions, 1);
@@ -481,6 +490,8 @@ test_xmp_regions_ns_prefix (void)
 
 	data = tracker_xmp_new (contents, size, "test://file");
 
+        g_free (contents);
+
 	g_assert_cmpint (2, ==, g_slist_length (data->regions));
 
 	region = g_slist_nth_data (data->regions, 1);
@@ -505,7 +516,7 @@ test_xmp_regions_ns_prefix (void)
 	tracker_xmp_free (data);
 }
 
-void
+static void
 test_xmp_regions_nb282393 ()
 {
 	TrackerXmpData *data;
@@ -524,6 +535,8 @@ test_xmp_regions_nb282393 ()
 
 	data = tracker_xmp_new (contents, size, "test://file");
 
+        g_free (contents);
+
 	g_assert_cmpint (1, ==, g_slist_length (data->regions));
 
 	/* Regions are stacked while parsing.*/
@@ -539,7 +552,7 @@ test_xmp_regions_nb282393 ()
 	tracker_xmp_free (data);
 }
 
-void
+static void
 test_xmp_regions_nb282393_2 ()
 {
 	TrackerXmpData *data;
@@ -558,6 +571,8 @@ test_xmp_regions_nb282393_2 ()
 
 	data = tracker_xmp_new (contents, size, "test://file");
 
+        g_free (contents);
+
 	g_assert_cmpint (1, ==, g_slist_length (data->regions));
 
 	/* Regions are stacked while parsing.*/
